# -*- coding:utf-8 -*-

import json
import logging
import time
import requests
import threading
from URMessageChannel import logconfig, TimerEvtHandle, init_base
from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTClient

#your application install path
APP_PATH = "/usr/pyuser/app/"
#your application name
MYAPP_NAME = "ug87_http_api_demo"
config = None

try:
    with open(APP_PATH + MYAPP_NAME + "/config.json", "r") as f:
        config = json.loads("".join(f.readlines()))
except Exception:
    raise

def get_api_token(username, password, serv_addr):
    req = requests.post("https://" + serv_addr + "/api/internal/login", data=json.dumps({"username":username, "password": password}), verify=False)
    if req.status_code == 200:
        return req.json()["jwt"]


class UpdateTimer(TimerEvtHandle):
    def __init__(self):
        self.base = init_base()
        self.api_server = config["http_api_config"]["api_server"]
        self.api_username = config["http_api_config"]["api_username"]
        self.api_password = config["http_api_config"]["api_password"]
        self.endpoint = config["aws_connect_config"]["endpoint"]
        self.ca_path = APP_PATH + MYAPP_NAME + "/" + config["aws_connect_config"]["root_ca"]
        self.cert_path = APP_PATH + MYAPP_NAME + "/" + config["aws_connect_config"]["tls_cert"]
        self.key_path = APP_PATH + MYAPP_NAME + "/" + config["aws_connect_config"]["tls_key"]
        self.use_websocket = config["aws_connect_config"]["use_websocket"]
        self.client_id = config["aws_connect_config"]["client_ID"]
        self.devices = [{"dev_eui": eui, "topic": config["aws_connect_config"]["uplink_topic_template"].replace("{eui}", eui)} for eui in config["http_api_config"]["device_euis"]]
        self.aws_mqtt_client = None
        self.connected = False
        self.api_token = None
        TimerEvtHandle.__init__(self, self.base, 1)

    def init_and_config_aws_mqtt_client(self):
        if self.use_websocket:
            self.aws_mqtt_client = AWSIoTMQTTClient(self.client_id, useWebsocket=True)
            self.aws_mqtt_client.configureEndpoint(self.endpoint, 443)
            self.aws_mqtt_client.configureCredentials(self.ca_path)
        else:
            self.aws_mqtt_client = AWSIoTMQTTClient(self.client_id)
            self.aws_mqtt_client.configureEndpoint(self.endpoint, 8883)
            self.aws_mqtt_client.configureCredentials(self.ca_path, self.key_path, self.cert_path)
        self.aws_mqtt_client.configureAutoReconnectBackoffTime(1, 32, 20)
        # Infinite offline Publish queueing
        self.aws_mqtt_client.configureOfflinePublishQueueing(-1)
        #Draining 2 Hz
        self.aws_mqtt_client.configureDrainingFrequency(2)
        self.aws_mqtt_client.configureConnectDisconnectTimeout(10)
        self.aws_mqtt_client.configureMQTTOperationTimeout(5)
        self.aws_mqtt_client.connect()
        self.connected = True

    def publish_to_aws(self, topic, msg):
        self.aws_mqtt_client.publish(topic, msg, 1)
        logging.info("publish {0} to aws".format(msg))

    def subscribe_to_aws(self):
        for dev in self.devices:
            self.aws_mqtt_client.subscribe(dev["topic"].encode("utf-8"), 1, self.sub_callback)
            logging.info("subscribe topic: {0}".format(dev["topic"]))

    def sub_callback(self, client, userdata, msg):
        logging.info("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
        logging.info("Receive a new message: {0} from topic: {1}".format(msg.payload, msg.topic))
        logging.info("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")

    def receive_uplink_data(self, token, eui, topic):
        url = "https://" + self.api_server + "/api/devices/" + eui + "/data"
        headers = {"Authorization": "Bearer " + token}
        req = requests.get(url, headers=headers, stream=True, verify=False)
        for line in req.iter_lines():
            if line:
                decoded_line = line.decode("utf-8")
                self.publish_to_aws(topic, decoded_line)

    def timerHandle(self, evt, userdata):
        while not self.connected:
            try:
                self.init_and_config_aws_mqtt_client()
            except:
                time.sleep(0.5)
                continue
        self.subscribe_to_aws()
        while self.api_token is None:
            try:
                self.api_token = get_api_token(self.api_username, self.api_password, self.api_server)
            except:
                time.sleep(0.5)
                continue
        thread_list = []
        for dev in self.devices:
            t = threading.Thread(target=self.receive_uplink_data, args=(self.api_token, dev["dev_eui"].encode("utf-8"), dev["topic"].encode("utf-8")))
            t.setDaemon(True)
            thread_list.append(t)
        for t in thread_list:
            t.start()
        for t in thread_list:
            t.join()
