# -*- coding:utf-8 -*-

import json
import logging
import time
import paho.mqtt.client as mqtt
from URMessageChannel import logconfig, TimerEvtHandle, init_base
from AWSIoTPythonSDK.MQTTLib import AWSIoTMQTTClient

#your application install path
APP_PATH = "/usr/pyuser/app/"
#your application name
MYAPP_NAME = "ug87_mqtt_api_demo"
config = None

try:
    with open(APP_PATH + MYAPP_NAME + "/config.json", "r") as f:
        config = json.loads("".join(f.readlines()))
except Exception:
    raise

def on_connect(mqttc, obj, flags, rc):
    logging.info("Connection {0}".format({0: "successful!",
        1: "refused - incorrect protocol version",
        2: "refused - invalid client identifier",
        3: "refused - server unavailable",
        4: "refused - bad username or password",
        5: "refused - not authorised"}[rc]))

def on_subscribe(mqttc, obj, mid, granted_qos):
    logging.info("Subscribed: {0} {1}".format(mid, granted_qos))

def get_lora_mqtt_client():
    mqttc = mqtt.Client()
    mqttc.on_connect = on_connect
    mqttc.on_subscribe = on_subscribe
    return mqttc


class UpdateTimer(TimerEvtHandle):
    def __init__(self):
        self.base = init_base()
        self.mqtt_server = config["mqtt_api_config"]["mqtt_server"]
        self.mqtt_username = config["mqtt_api_config"]["mqtt_username"]
        self.mqtt_password = config["mqtt_api_config"]["mqtt_password"]
        self.endpoint = config["aws_connect_config"]["endpoint"]
        self.ca_path = APP_PATH + MYAPP_NAME + "/" + config["aws_connect_config"]["root_ca"]
        self.cert_path = APP_PATH + MYAPP_NAME + "/" + config["aws_connect_config"]["tls_cert"]
        self.key_path = APP_PATH + MYAPP_NAME + "/" + config["aws_connect_config"]["tls_key"]
        self.use_websocket = config["aws_connect_config"]["use_websocket"]
        self.client_id = config["aws_connect_config"]["client_ID"]
        self.devices = [{"dev_eui": dev["eui"], 
            "lora_topic": config["mqtt_api_config"]["device_uplinktopic_template"].replace("{appID}", dev["app_ID"]).replace("{devEUI}", dev["eui"]), 
            "aws_topic": config["aws_connect_config"]["uplink_topic_template"].replace("{eui}", dev["eui"])} for dev in config["mqtt_api_config"]["devices"]]
        self.aws_mqtt_client = None
        self.lora_mqtt_client = None
        self.connected = False
        TimerEvtHandle.__init__(self, self.base, 1)

    def init_and_config_aws_mqtt_client(self):
        if self.use_websocket:
            self.aws_mqtt_client = AWSIoTMQTTClient(self.client_id, useWebsocket=True)
            self.aws_mqtt_client.configureEndpoint(self.endpoint, 443)
            self.aws_mqtt_client.configureCredentials(self.ca_path)
        else:
            self.aws_mqtt_client = AWSIoTMQTTClient(self.client_id)
            self.aws_mqtt_client.configureEndpoint(self.endpoint, 8883)
            self.aws_mqtt_client.configureCredentials(self.ca_path, self.key_path, self.cert_path)
        self.aws_mqtt_client.configureAutoReconnectBackoffTime(1, 32, 20)
        # Infinite offline Publish queueing
        self.aws_mqtt_client.configureOfflinePublishQueueing(-1)
        #Draining 2 Hz
        self.aws_mqtt_client.configureDrainingFrequency(2)
        self.aws_mqtt_client.configureConnectDisconnectTimeout(10)
        self.aws_mqtt_client.configureMQTTOperationTimeout(5)
        self.aws_mqtt_client.connect()
        self.connected = True

    def publish_to_aws(self, topic, msg):
        self.aws_mqtt_client.publish(topic, msg, 1)
        logging.info("publish {0} to aws".format(msg))

    def subscribe_to_aws(self):
        for dev in self.devices:
            self.aws_mqtt_client.subscribe(dev["aws_topic"].encode("utf-8"), 1, self.sub_callback)
            logging.info("subscribe topic: {0}".format(dev["aws_topic"]))

    def sub_callback(self, client, userdata, msg):
        logging.info("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")
        logging.info("Receive a new message: {0} from topic: {1}".format(msg.payload, msg.topic))
        logging.info("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~")

    def loramqtt_on_message(self, mqttc, obj, msg):
        for dev in self.devices:
            if dev["lora_topic"].encode("utf-8") == msg.topic.encode("utf-8"):
                self.publish_to_aws(dev["aws_topic"].encode("utf-8"), msg.payload)
                break

    def timerHandle(self, evt, userdata):
        while not self.connected:
            try:
                self.init_and_config_aws_mqtt_client()
            except:
                time.sleep(0.5)
                continue
        self.subscribe_to_aws()
        self.lora_mqtt_client = get_lora_mqtt_client()
        self.lora_mqtt_client.on_message = self.loramqtt_on_message
        self.lora_mqtt_client.username_pw_set(self.mqtt_username, self.mqtt_password)
        self.lora_mqtt_client.connect(self.mqtt_server.split(":")[0], port=int(self.mqtt_server.split(":")[1]))
        all_topic_qos = [(dev["lora_topic"], 0) for dev in self.devices]
        self.lora_mqtt_client.subscribe(all_topic_qos)
        self.lora_mqtt_client.loop_forever()
